//---------------------------------------------------------------------------
//
// Evj        Model the evolution of DNA binding site recognition
//
// Module:    Base, a simple class with stuff about bases
// Author:    Paul C. Anagnostopoulos
// Created:   1 April 2005
//
// This computer program fits the description in the U.S. Copyright Act
// of a "United States Government work." It was written as a part of
// my/our official duties as a Government employee. This means it cannot
// be copyrighted. The program is freely available for use without a
// copyright notice, and there are no restrictions on its use, now or
// subsequently. We retain no rights in the program. Please acknowledge
// the source of this code in any releases you make.
//
//---------------------------------------------------------------------------


import java.awt.*;
import java.util.Random;

//                      DNA Bases
//                      --- -----


// This class provides a few simple things to help with DNA bases.
// The bases are represented by the integers 0, 1, 2, and 3.

public class Base
{
  // Here are the public constants A, C, G, and T.
  
  public static final byte A = 0, C = 1, G = 2, T = 3;

  // Return the letter corresponding to a base.

  private static final String baseLetter[] = {"A", "C", "G", "T"};

  public static String letter (int b)
  {
    return baseLetter[b];
  }

  // Return the "standard" display color for a base.

  private static final Color baseColor[] = {
    new Color(0.0f, 1.0f, 0.0f),
    new Color(0.0f, 0.5f, 1.0f),
    new Color(1.0f, 0.7f, 0.0f),
    new Color(0.8f, 0.0f, 0.0f)};

  public static Color color (int b)
  {
    return baseColor[b];
  }

  // Return a random base using the provided generator.

  public static byte random (Random generator)
  {
    return (byte)generator.nextInt(4);
  }
}
