//---------------------------------------------------------------------------
//
// Evj        Model the evolution of DNA binding site recognition
//
//
// Module:    MainFrame, the GUI code for the main Evj window
// Author:    Paul C. Anagnostopoulos
// Created:   3 April 2005
//            The bulk of this code was generated by JFrameBuilder.
//
// This computer program fits the description in the U.S. Copyright Act
// of a "United States Government work." It was written as a part of
// my/our official duties as a Government employee. This means it cannot
// be copyrighted. The program is freely available for use without a
// copyright notice, and there are no restrictions on its use, now or
// subsequently. We retain no rights in the program. Please acknowledge
// the source of this code in any releases you make.
//
//---------------------------------------------------------------------------

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.text.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

//                      Main Frame
//                      ---- -----


public class MainFrame extends JFrame implements NexusObserver
{
  // This public variable will contain the font to be used in all
  // fields, spinners, etc.
  
  public static Font fieldFont;

  public static final Color backgroundColor = new Color(0.97f, 0.97f, 0.97f);
  
  private static final Color runningColor    = new Color(59, 198, 72),
                             pausedColor     = Color.red;
  
  // Variables declaration
	private JLabel speedLabel;
	private JLabel sitesLabel;
  private JLabel sites2Label;
	private JLabel widthLabel;
  private JLabel mutationsLabel;
	private JLabel rfreqLabel;
  private JLabel idLabel;
  private JLabel mistakesLabel;
  private JLabel mistakes2Label;
	private JLabel bestLabel;
	private JLabel worstLabel;
	private JLabel generationLabel;
	private JLabel cyclesLabel;
  private JProgressBar progressBar;
	private JLabel rseqLabel;
  private JLabel rseq2Label;
	private JLabel populationLabel;
  private JLabel potentialLabel;
  private JLabel population2Label;
  private JLabel potential2Label;
  private JLabel width2Label;
  private JLabel mutations2Label;
  private JLabel rfreq2Label;
  private JLabel updateLabel;
  private JLabel update2Label;
	private JToggleButton runButton;
	private JTextField sitesField;
	private JTextField widthField;
  private JTextField mutationsField;
	private JTextField rfreqField;
  private JCheckBox selectionCheckBox;
  private JCheckBox perfectionCheckBox;
  private JCheckBox convergenceCheckBox;
  private JTextField bestIDField;
  private JTextField worstIDField;
  private JTextField bestMistakesField;
	private JTextField worstMistakesField;
	private JTextField generationField;
	private JSpinner cyclesSpinner;
	private JTextField bestRseqField;
	private JTextField worstRseqField;
	private JTextField populationField;
  private JTextField potentialField;
	private JSpinner speedSpinner;
	private JSpinner updateSpinner;
	private JSeparator leftSeparator;
  private JSeparator rightSeparator;
	private JTabbedPane displayTabbedPane;
	private JButton stepButton;
  private IconicLogoPanel logoPanel;
	private JPanel contentPane;
  private JButton restartButton;
  private JButton newButton;
	private JButton helpButton;
	private JButton aboutButton;
	private JPanel buttonBar;

	private JScrollPane geneSeqScrollPane;
  private GeneSeqPanel geneSeqPanel;
  private JLabel geneSeqTitleLabel;
  private JViewport geneSeqVP;

  private JScrollPane seqLogoScrollPane;
  private SeqLogoPanel seqLogoPanel;
  private JLabel seqLogoTitleLabel;
  private JViewport seqLogoVP;

  private JScrollPane graphScrollPane;
  private GraphPanel graphPanel;
  private JLabel graphTitleLabel;
  private JViewport graphVP;

  private JScrollPane dataStatScrollPane;
  private DataStatPanel dataStatPanel;
  private JLabel dataStatTitleLabel;
  private JViewport dataStatVP;

  // End of variables declaration

	public MainFrame (boolean application)
	{
		super();
		initializeComponent();
		if (application)
      this.setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		this.setVisible(true);
	}

	private void initializeComponent()
	{
    // Set up a resize listener.

    this.addComponentListener(new ComponentAdapter () {
      public void componentResized (ComponentEvent e)
      {
        sizeTabbedPane();
      }});

      // Set up a window close listener.
      
      this.addWindowListener(new WindowAdapter () {
      public void windowClosing (WindowEvent e)
      {
        closeMainFrame();
      }});

    speedLabel = new JLabel();
		sitesLabel = new JLabel();
    sites2Label = new JLabel();
		widthLabel = new JLabel();
    mutationsLabel = new JLabel();
		rfreqLabel = new JLabel();
		idLabel = new JLabel();
    mistakesLabel = new JLabel();
    mistakes2Label = new JLabel();
		bestLabel = new JLabel();
		worstLabel = new JLabel();
		generationLabel = new JLabel();
		cyclesLabel = new JLabel();
    progressBar = new JProgressBar();
		updateLabel = new JLabel();
		rseqLabel = new JLabel();
    rseq2Label = new JLabel();
		populationLabel = new JLabel();
    potentialLabel = new JLabel();
    population2Label = new JLabel();
    potential2Label = new JLabel();
    width2Label = new JLabel();
    mutations2Label = new JLabel();
    rfreq2Label = new JLabel();
    update2Label = new JLabel();
		runButton = new JToggleButton();
		sitesField = new JTextField();
		widthField = new JTextField();
    mutationsField = new JTextField();
		rfreqField = new JTextField();
    selectionCheckBox = new JCheckBox();
    perfectionCheckBox = new JCheckBox();
    convergenceCheckBox = new JCheckBox();
    bestIDField = new JTextField();
    worstIDField = new JTextField();
    bestMistakesField = new JTextField();
		worstMistakesField = new JTextField();
		generationField = new JTextField();
		cyclesSpinner = new JSpinner();
		bestRseqField = new JTextField();
		worstRseqField = new JTextField();
		populationField = new JTextField();
    potentialField = new JTextField();
		speedSpinner = new JSpinner();
		updateSpinner = new JSpinner();
		leftSeparator = new JSeparator();
    rightSeparator = new JSeparator();
		displayTabbedPane = new JTabbedPane();
		stepButton = new JButton();
    logoPanel = new IconicLogoPanel();
		contentPane = (JPanel)this.getContentPane();
		restartButton = new JButton();
    newButton = new JButton();
		helpButton = new JButton();
		aboutButton = new JButton();
		buttonBar = new JPanel();

    geneSeqScrollPane = new JScrollPane();
    geneSeqPanel = new GeneSeqPanel(geneSeqScrollPane);
    geneSeqPanel.setBackground(backgroundColor);
    geneSeqPanel.setLayout(null);
    geneSeqScrollPane.setViewportView(geneSeqPanel);
    geneSeqTitleLabel = new JLabel();

    seqLogoScrollPane = new JScrollPane();
    seqLogoPanel = new SeqLogoPanel(seqLogoScrollPane);
    seqLogoPanel.setBackground(backgroundColor);
    seqLogoPanel.setLayout(null);
    seqLogoScrollPane.setViewportView(seqLogoPanel);
    seqLogoTitleLabel = new JLabel();

    graphScrollPane = new JScrollPane();
    graphPanel = new GraphPanel(graphScrollPane);
    graphPanel.setBackground(backgroundColor);
    graphPanel.setLayout(null);
    graphScrollPane.setViewportView(graphPanel);
    graphTitleLabel = new JLabel();

    dataStatScrollPane = new JScrollPane();
    dataStatPanel = new DataStatPanel(dataStatScrollPane);
    dataStatPanel.setBackground(backgroundColor);
    dataStatPanel.setLayout(null);
    dataStatScrollPane.setViewportView(dataStatPanel);
    dataStatTitleLabel = new JLabel();

    // Create a font to use in all the fields and spinners. It is
    // translated to account for the strange positioning of the text.
    // Make the font bold.
    
    AffineTransform trans = new AffineTransform();
    trans.setToTranslation(0.0, 3.0);
    fieldFont = sitesField.getFont().deriveFont(Font.BOLD, trans);

		// speedLabel

		speedLabel.setText("Speed");
		speedLabel.setVerticalAlignment(SwingConstants.TOP);
    speedLabel.setToolTipText(Resource.get("mftip.speedSpinner"));

		// sitesLabel

		sitesLabel.setText("Binding sites");
		sitesLabel.setVerticalAlignment(SwingConstants.TOP);
    sitesLabel.setToolTipText(Resource.get("mftip.sitesField"));

    // sites2Label

    sites2Label.setText("count");
    sites2Label.setVerticalAlignment(SwingConstants.TOP);
    sites2Label.setToolTipText(Resource.get("mftip.sitesField"));
    
		// widthLabel

		widthLabel.setText("Site width");
		widthLabel.setVerticalAlignment(SwingConstants.TOP);
    widthLabel.setToolTipText(Resource.get("mftip.widthField"));

   // mutationsLabel

    mutationsLabel.setText("Mutations");
    mutationsLabel.setVerticalAlignment(SwingConstants.TOP);
    mutationsLabel.setToolTipText(Resource.get("mftip.mutationsField"));

		// rfreqLabel
		//
		rfreqLabel.setText("Rfrequency");
		rfreqLabel.setVerticalAlignment(SwingConstants.TOP);
    rfreqLabel.setToolTipText(Resource.get("mftip.rfreqField"));

		// idLabel

		idLabel.setText("ID");
		idLabel.setVerticalAlignment(SwingConstants.TOP);
    idLabel.setToolTipText(Resource.get("mftip.idLabel"));

    // mistakesLabel

    mistakesLabel.setText("Mistakes");
    mistakesLabel.setVerticalAlignment(SwingConstants.TOP);
    mistakesLabel.setToolTipText(Resource.get("mftip.mistakesLabel"));

    // mistakes2Label

    mistakes2Label.setText("points");
    mistakes2Label.setVerticalAlignment(SwingConstants.TOP);
    mistakes2Label.setToolTipText(Resource.get("mftip.mistakesLabel"));

		// bestLabel

		bestLabel.setHorizontalAlignment(SwingConstants.CENTER);
		bestLabel.setText("Best");
		bestLabel.setVerticalAlignment(SwingConstants.TOP);
    bestLabel.setToolTipText(Resource.get("mftip.bestLabel"));

		// worstLabel
		//
		worstLabel.setHorizontalAlignment(SwingConstants.CENTER);
		worstLabel.setText("Worst");
		worstLabel.setVerticalAlignment(SwingConstants.TOP);
    worstLabel.setToolTipText(Resource.get("mftip.worstLabel"));

		// generationLabel
		//
		generationLabel.setText("Generation");
		generationLabel.setVerticalAlignment(SwingConstants.TOP);
    generationLabel.setToolTipText(Resource.get("mftip.generationField"));

		// cyclesLabel

		cyclesLabel.setText("Cycles to run");
		cyclesLabel.setVerticalAlignment(SwingConstants.TOP);
    cyclesLabel.setToolTipText(Resource.get("mftip.cyclesSpinner"));

    // progressBar

    progressBar.setForeground(runningColor);
    progressBar.setStringPainted(true);
    progressBar.setToolTipText(Resource.get("mftip.progressBar"));
    progressBar.setFont(fieldFont);
    
    // updateLabel

		updateLabel.setText("Update every");
		updateLabel.setVerticalAlignment(SwingConstants.TOP);
    updateLabel.setToolTipText(Resource.get("mftip.updateSpinner"));

		// rseqLabel

		rseqLabel.setText("Rsequence");
		rseqLabel.setVerticalAlignment(SwingConstants.TOP);
    rseqLabel.setToolTipText(Resource.get("mftip.rseqLabel"));

    // rseq2Label

    rseq2Label.setText("bits");
    rseq2Label.setVerticalAlignment(SwingConstants.TOP);
    rseq2Label.setToolTipText(Resource.get("mftip.rseqLabel"));

		// populationLabel
		//
		populationLabel.setText("Population");
		populationLabel.setVerticalAlignment(SwingConstants.TOP);
    populationLabel.setToolTipText(Resource.get("mftip.populationField"));

    // potentialLabel
    //
    potentialLabel.setText("Potential sites");
    potentialLabel.setVerticalAlignment(SwingConstants.TOP);
    potentialLabel.setToolTipText(Resource.get("mftip.potentialField"));
    //
    // population2Label
    //
    population2Label.setText("creatures");
    population2Label.setVerticalAlignment(SwingConstants.TOP);
    population2Label.setToolTipText(Resource.get("mftip.populationField"));

    // potential2Label
    //
    potential2Label.setText("bases");
    potential2Label.setVerticalAlignment(SwingConstants.TOP);
    potential2Label.setToolTipText(Resource.get("mftip.potentialField"));

    // width2Label
    //
    width2Label.setText("bases");
    width2Label.setVerticalAlignment(SwingConstants.TOP);
    width2Label.setToolTipText(Resource.get("mftip.widthField"));

    // mutations2Label
    //
    mutations2Label.setText("per generation");
    mutations2Label.setVerticalAlignment(SwingConstants.TOP);
    mutations2Label.setToolTipText(Resource.get("mftip.mutationsField"));

    // rfreq2Label

    rfreq2Label.setText("bits");
    rfreq2Label.setVerticalAlignment(SwingConstants.TOP);
    rfreq2Label.setToolTipText(Resource.get("mftip.rfreqField"));

    // update2Label
    //
    update2Label.setText("generations");
    update2Label.setVerticalAlignment(SwingConstants.TOP);
    update2Label.setToolTipText(Resource.get("mftip.updateSpinner"));

		// runButton
		//
		runButton.setText("Run");
		runButton.setVerticalAlignment(SwingConstants.TOP);
    runButton.setToolTipText(Resource.get("mftip.runButton"));
    runButton.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e)
			{
				runButton_itemStateChanged(e);
			}});

		// sitesField

    sitesField.setHorizontalAlignment(JTextField.RIGHT);
		sitesField.setEditable(false);
		sitesField.setFocusable(false);
    sitesField.setToolTipText(Resource.get("mftip.sitesField"));
    sitesField.setFont(fieldFont);

		// widthField

    widthField.setHorizontalAlignment(JTextField.RIGHT);
		widthField.setEditable(false);
		widthField.setFocusable(false);
    widthField.setToolTipText(Resource.get("mftip.widthField"));
    widthField.setFont(fieldFont);

    // mutationsField

    mutationsField.setHorizontalAlignment(JTextField.RIGHT);
    mutationsField.setEditable(false);
    mutationsField.setFocusable(false);
    mutationsField.setToolTipText(Resource.get("mftip.mutationsField"));
    mutationsField.setFont(fieldFont);

		// rfreqField

    rfreqField.setHorizontalAlignment(JTextField.RIGHT);
		rfreqField.setEditable(false);
		rfreqField.setFocusable(false);
    rfreqField.setToolTipText(Resource.get("mftip.rfreqField"));
    rfreqField.setFont(fieldFont);

    // selectionCheckBox

    selectionCheckBox.setBackground(backgroundColor);
    selectionCheckBox.setText("Perform selection");
    selectionCheckBox.setVerticalAlignment(SwingConstants.TOP);
    selectionCheckBox.setVerticalTextPosition(SwingConstants.TOP);
    selectionCheckBox.setToolTipText(Resource.get("mftip.selectionCheckBox"));
    selectionCheckBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e)
      {
        selectionCheckBox_itemStateChanged(e);
      }});

    // perfectionCheckBox

    perfectionCheckBox.setBackground(backgroundColor);
    perfectionCheckBox.setText("Pause on perfect creature");
    perfectionCheckBox.setVerticalAlignment(SwingConstants.TOP);
    perfectionCheckBox.setVerticalTextPosition(SwingConstants.TOP);
    perfectionCheckBox.setToolTipText(Resource.get("mftip.perfectionCheckBox"));
    perfectionCheckBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e)
      {
        perfectionCheckBox_itemStateChanged(e);
      }});

    // convergenceCheckBox

    convergenceCheckBox.setBackground(backgroundColor);
    convergenceCheckBox.setText("Pause on Rseq >= Rfreq");
    convergenceCheckBox.setVerticalAlignment(SwingConstants.TOP);
    convergenceCheckBox.setVerticalTextPosition(SwingConstants.TOP);
    convergenceCheckBox.setToolTipText(Resource.get("mftip.convergenceCheckBox"));
    convergenceCheckBox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e)
      {
        convergenceCheckBox_itemStateChanged(e);
      }});

    // bestIDField

    bestIDField.setHorizontalAlignment(JTextField.RIGHT);
		bestIDField.setEditable(false);
    bestIDField.setToolTipText(Resource.get("mftip.bestIDField"));
    bestIDField.setFont(fieldFont);

    // worstIDField

    worstIDField.setHorizontalAlignment(JTextField.RIGHT);
    worstIDField.setEditable(false);
    worstIDField.setToolTipText(Resource.get("mftip.worstIDField"));
    worstIDField.setFont(fieldFont);

    // bestMistakesField

    bestMistakesField.setHorizontalAlignment(JTextField.RIGHT);
    bestMistakesField.setEditable(false);
    bestMistakesField.setToolTipText(Resource.get("mftip.bestMistakesField"));
    bestMistakesField.setFont(fieldFont);

		// worstMistakesField

    worstMistakesField.setHorizontalAlignment(JTextField.RIGHT);
		worstMistakesField.setEditable(false);
    worstMistakesField.setToolTipText(Resource.get("mftip.worstMistakesField"));
    worstMistakesField.setFont(fieldFont);

		// generationField

    generationField.setHorizontalAlignment(JTextField.RIGHT);
		generationField.setEditable(false);
		generationField.setFocusable(false);
    generationField.setToolTipText(Resource.get("mftip.generationField"));
    generationField.setFont(fieldFont);

		// cyclesSpinner

    SpinnerModel model = cyclesSpinner.getModel();
    ((SpinnerNumberModel)model).setMinimum(new Integer(1));
    ((SpinnerNumberModel)model).setStepSize(new Integer(10000));
    cyclesSpinner.setToolTipText(Resource.get("mftip.cyclesSpinner"));
    cyclesSpinner.setFont(fieldFont);
    cyclesSpinner.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e)
      {
        cyclesSpinner_stateChanged(e);
      }});

		// bestRseqField

    bestRseqField.setHorizontalAlignment(JTextField.RIGHT);
		bestRseqField.setEditable(false);
    bestRseqField.setToolTipText(Resource.get("mftip.bestRseqField"));
    bestRseqField.setFont(fieldFont);

		// worstRseqField

    worstRseqField.setHorizontalAlignment(JTextField.RIGHT);
		worstRseqField.setEditable(false);
    worstRseqField.setToolTipText(Resource.get("mftip.worstRseqField"));
    worstRseqField.setFont(fieldFont);

		// populationField

    populationField.setHorizontalAlignment(JTextField.RIGHT);
		populationField.setEditable(false);
		populationField.setFocusable(false);
    populationField.setToolTipText(Resource.get("mftip.populationField"));
    populationField.setFont(fieldFont);

    // potentialField

    potentialField.setHorizontalAlignment(JTextField.RIGHT);
    potentialField.setEditable(false);
    potentialField.setFocusable(false);
    potentialField.setToolTipText(Resource.get("mftip.potentialField"));
    potentialField.setFont(fieldFont);

		// speedSpinner

    model = speedSpinner.getModel();
    ((SpinnerNumberModel)model).setMinimum(new Integer(1));
    ((SpinnerNumberModel)model).setMaximum(new Integer(21));
    speedSpinner.setValue(new Integer(5));
    speedSpinner.setToolTipText(Resource.get("mftip.speedSpinner"));
    speedSpinner.setFont(fieldFont);
		speedSpinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e)
			{
				speedSpinner_stateChanged(e);
			}});

		// updateSpinner

    model = updateSpinner.getModel();
    ((SpinnerNumberModel)model).setMinimum(new Integer(1));
    ((SpinnerNumberModel)model).setMaximum(new Integer(100000));
    updateSpinner.setValue(new Integer(1));
    updateSpinner.setToolTipText(Resource.get("mftip.updateSpinner"));
    updateSpinner.setFont(fieldFont);
		updateSpinner.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e)
			{
				updateSpinner_stateChanged(e);
			}});

		// Separators

		leftSeparator.setOrientation(SwingConstants.VERTICAL);
    rightSeparator.setOrientation(SwingConstants.VERTICAL);

		// displayTabbedPane

    displayTabbedPane.addTab("Genetic sequence", geneSeqScrollPane);
    displayTabbedPane.setToolTipTextAt(0, Resource.get("mftip.geneSeqTab"));
    displayTabbedPane.addTab("Sequence logo", seqLogoScrollPane);
    displayTabbedPane.setToolTipTextAt(1, Resource.get("tip.NYI"));
    displayTabbedPane.addTab("Graphs", graphScrollPane);
    displayTabbedPane.setToolTipTextAt(2, Resource.get("tip.NYI"));
		displayTabbedPane.addTab("Data & Statistics", dataStatScrollPane);
    displayTabbedPane.setToolTipTextAt(3, Resource.get("tip.NYI"));

		// stepButton

		stepButton.setText("Step");
		stepButton.setVerticalAlignment(SwingConstants.TOP);
    stepButton.setToolTipText(Resource.get("mftip.stepButton"));
		stepButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				stepButton_actionPerformed(e);
			}});

    // logoPanel

    logoPanel.setLayout(null);
    logoPanel.setOpaque(false);
    logoPanel.setFocusable(false);
    logoPanel.setToolTipText(Resource.get("mftip.logoPanel"));

    // contentPane

    contentPane.setBackground(backgroundColor);
    contentPane.setLayout(null);
    addComponent(contentPane, speedLabel, 21,110,44,18);
    addComponent(contentPane, generationLabel, 18,155,75,21);
    addComponent(contentPane, cyclesLabel, 18,195,83,22);
    addComponent(contentPane, progressBar, 17,235,170,18);
    addComponent(contentPane, populationLabel, 232,65,73,18);
    addComponent(contentPane, populationField, 325,63,50,22);
    addComponent(contentPane, population2Label, 382,65,60,18);
    addComponent(contentPane, potentialLabel, 232,99,89,18);
    addComponent(contentPane, potentialField, 324,96,50,22);
    addComponent(contentPane, potential2Label, 382,99,60,18);
    addComponent(contentPane, sitesLabel, 232,127,78,18);
    addComponent(contentPane, sitesField, 325,126,50,22);
    addComponent(contentPane, sites2Label, 382,127,60,18);
    addComponent(contentPane, widthLabel, 232,156,67,18);
    addComponent(contentPane, widthField, 323,157,50,22);
    addComponent(contentPane, width2Label, 382,158,60,18);
    addComponent(contentPane, mutationsLabel, 232,190,67,18);
    addComponent(contentPane, mutationsField, 323,187,50,22);
    addComponent(contentPane, mutations2Label, 382,190,100,18);
    addComponent(contentPane, selectionCheckBox, 228,218,140,24);
    addComponent(contentPane, runButton, 13,63,83,28);

    addComponent(contentPane, bestLabel, 802,60,60,18);
    addComponent(contentPane, worstLabel, 871,60,60,20);
    addComponent(contentPane, idLabel, 727,85,60,18);
    addComponent(contentPane, mistakesLabel, 727,115,60,18);
    addComponent(contentPane, mistakes2Label, 940,115,60,18);
    addComponent(contentPane, rseqLabel, 727,144,68,18);
    addComponent(contentPane, rseq2Label, 940,145,68,18);
    addComponent(contentPane, bestIDField, 804,83,54,22);
    addComponent(contentPane, worstIDField, 873,83,54,22);
    addComponent(contentPane, bestMistakesField, 804,112,54,22);
    addComponent(contentPane, worstMistakesField, 873,112,54,22);
    addComponent(contentPane, bestRseqField, 804,142,54,22);
    addComponent(contentPane, worstRseqField, 873,142,54,22);
    addComponent(contentPane, rfreqLabel, 727,180,75,18);
    addComponent(contentPane, rfreqField, 804,177,54,22);
    addComponent(contentPane, rfreq2Label, 940,180,60,18);
    addComponent(contentPane, perfectionCheckBox, 718,218,180,24);
    addComponent(contentPane, convergenceCheckBox, 718,242,180,24);
    addComponent(contentPane, updateLabel, 905,215,75,17);
    addComponent(contentPane, update2Label, 905,266,75,22);

    addComponent(contentPane, generationField, 105,154,63,22);
    addComponent(contentPane, cyclesSpinner, 105,195,79,22);
    addComponent(contentPane, speedSpinner, 105,108,70,22);
    addComponent(contentPane, updateSpinner, 905,240,65,22);
    addComponent(contentPane, leftSeparator, 208,58,10,200);
    addComponent(contentPane, stepButton, 102,63,83,28);
    addComponent(contentPane, buttonBar, 10,5,975,38);
    addComponent(contentPane, logoPanel, 485,50,215,205);
    addComponent(contentPane, displayTabbedPane, 6,270,982,599);

    JScrollBar bar;
    
    // geneSeqScrollPane

    geneSeqScrollPane.setHorizontalScrollBarPolicy(
                      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    geneSeqScrollPane.setVerticalScrollBarPolicy(
                      ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    bar = geneSeqScrollPane.getVerticalScrollBar();
    bar.setUnitIncrement(20);
    geneSeqVP = new JViewport();
    geneSeqVP.setView(geneSeqTitleLabel);
    geneSeqScrollPane.setColumnHeader(geneSeqVP);

    // geneSeqTitleLabel

    geneSeqTitleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    geneSeqTitleLabel.setVerticalAlignment(SwingConstants.TOP);

    // seqLogoPanel

    seqLogoScrollPane.setHorizontalScrollBarPolicy(
                      ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    seqLogoScrollPane.setVerticalScrollBarPolicy(
                      ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    bar = seqLogoScrollPane.getVerticalScrollBar();
    bar.setUnitIncrement(20);
    seqLogoVP = new JViewport();
    seqLogoVP.setView(seqLogoTitleLabel);
    seqLogoScrollPane.setColumnHeader(seqLogoVP);

    // seqLogoTitleLabel

    seqLogoTitleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    seqLogoTitleLabel.setVerticalAlignment(SwingConstants.TOP);

    // graphScrollPane

    graphScrollPane.setHorizontalScrollBarPolicy(
                    ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    graphScrollPane.setVerticalScrollBarPolicy(
                    ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    bar = graphScrollPane.getVerticalScrollBar();
    bar.setUnitIncrement(20);
    graphVP = new JViewport();
    graphVP.setView(graphTitleLabel);
    graphScrollPane.setColumnHeader(graphVP);

    // graphTitleLabel

    graphTitleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    graphTitleLabel.setVerticalAlignment(SwingConstants.TOP);

    // geneSeqScrollPane

    dataStatScrollPane.setHorizontalScrollBarPolicy(
                        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    dataStatScrollPane.setVerticalScrollBarPolicy(
                        ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
    bar = dataStatScrollPane.getVerticalScrollBar();
    bar.setUnitIncrement(20);
    dataStatVP = new JViewport();
    dataStatVP.setView(dataStatTitleLabel);
    dataStatScrollPane.setColumnHeader(dataStatVP);

    // dataStatTitleLabel

    dataStatTitleLabel.setHorizontalAlignment(SwingConstants.CENTER);
    dataStatTitleLabel.setVerticalAlignment(SwingConstants.TOP);

		// restartButton
		//
		restartButton.setText("Restart");
		restartButton.setVerticalAlignment(SwingConstants.TOP);
    restartButton.setToolTipText(Resource.get("mftip.restartButton"));
		restartButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				restartButton_actionPerformed(e);
			}});

    //
    // newButton
    //
    newButton.setText("New");
    newButton.setVerticalAlignment(SwingConstants.TOP);
    newButton.setToolTipText(Resource.get("mftip.newButton"));
    newButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        newButton_actionPerformed(e);
      }});

		// helpButton

		helpButton.setText("Help");
		helpButton.setVerticalAlignment(SwingConstants.TOP);
    helpButton.setToolTipText(Resource.get("mftip.helpButton"));
		helpButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				helpButton_actionPerformed(e);
			}});

		// aboutButton

		aboutButton.setText("About");
		aboutButton.setVerticalAlignment(SwingConstants.TOP);
    aboutButton.setToolTipText(Resource.get("mftip.aboutButton"));
		aboutButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e)
			{
				aboutButton_actionPerformed(e);
			}});

		// buttonBar
		//
		buttonBar.setLayout(null);
		buttonBar.setBorder(BorderFactory.createEtchedBorder());
		buttonBar.setBackground(new Color(129, 176, 140));
		addComponent(buttonBar, restartButton,  12,6,83,28);
    addComponent(buttonBar, newButton,     124,6,83,28);
		addComponent(buttonBar, helpButton,    791,6,83,28);
		addComponent(buttonBar, aboutButton,   880,5,83,28);

		// MainFrame: set the title and icon; location; size.

    this.setIconImage(
      new ImageIcon(getClass().getResource("Evj-trex1-icon.jpg")).getImage());
    this.setLocation(new Point(12, 29));
    this.setSize(new Dimension(1005, 893));
	}

	private void addComponent (Container container, Component c,
                             int x, int y, int width, int height)
	{
		c.setBounds(x, y, width, height);
		container.add(c);
	}

//                      MainFrame Helpers
//                      --------- -------


  private void closeMainFrame ()
  {
    Evj.killSimulation();
  }
  
  private void sizeTabbedPane ()
  {
    displayTabbedPane.setSize(
      this.getWidth() - 3 * displayTabbedPane.getX(),
      this.getHeight() - displayTabbedPane.getY() - 65);
  }

//                      Component Event Handlers
//                      --------- ----- --------


  private void aboutButton_actionPerformed(ActionEvent e)
  {
    AboutBox.display(this);
  }

  private void cyclesSpinner_stateChanged(ChangeEvent e)
  {
    cycles = ((Integer)cyclesSpinner.getValue()).intValue();
    Evj.setCycles(cycles);
  }

  private void helpButton_actionPerformed(ActionEvent e)
  {
    Msg.warning(this, Resource.get("mfmsg.help"));
    ///    Evj.openHelpBrowser();
  }

  private void newButton_actionPerformed(ActionEvent e)
  {
    popRunButton();
    Evj.setSpeed(Evj.pause);
    new NewDialog(this);
  }

  private void perfectionCheckBox_itemStateChanged(ItemEvent e)
  {
    Evj.setPerfection(e.getStateChange() == ItemEvent.SELECTED);
  }

  private void convergenceCheckBox_itemStateChanged(ItemEvent e)
  {
    Evj.setConvergence(e.getStateChange() == ItemEvent.SELECTED);
  }

	private void runButton_itemStateChanged(ItemEvent e)
	{
    if (e.getStateChange() == ItemEvent.SELECTED) {
      if (generation >= cycles) {
        popRunButton();
        Msg.warning(this, Resource.get("mfmsg.done"));
        return;
      }
      runButton.setText("Pause");
      Evj.setSpeed(((Integer)speedSpinner.getValue()).intValue());

    } else {
      runButton.setText("Run");
      Evj.setSpeed(Evj.pause);
    }    
	}

  private void restartButton_actionPerformed(ActionEvent e)
  {
    popRunButton();
    Evj.restartSimulation();
  }

  private void selectionCheckBox_itemStateChanged(ItemEvent e)
  {
    Evj.setSelection(e.getStateChange() == ItemEvent.SELECTED);
  }

  private void speedSpinner_stateChanged(ChangeEvent e)
  {
    ButtonModel model = runButton.getModel();
    if (model.isSelected())
      Evj.setSpeed(((Integer)speedSpinner.getValue()).intValue());
  }

  private void stepButton_actionPerformed(ActionEvent e)
  {
    popRunButton();
    if (generation >= cycles) {
      Msg.warning(this, Resource.get("mfmsg.done"));
      return;
    }
    Evj.setSpeed(Evj.step);
  }

	private void updateSpinner_stateChanged(ChangeEvent e)
	{
    Evj.setUpdateFreq(((Integer)updateSpinner.getValue()).intValue());
	}

//                      Display Update
//                      ------- ------


  private static final DecimalFormat  // Formatter for 2 decimal places.
    format0_00 = new DecimalFormat("0.00");

  private int generation;             // Generation number from the nexus.
  private int cycles;                 // Cycle count from the nexus.
  
  
  // This method is invoked by the simulator to update the main display
  // with the current state of the simulation, which is passed in the
  // nexus.
  
  public void acceptNexus (Nexus nexus)
  {
    // Set the window title.
    
    this.setTitle(" " + Evj.facilityName + " " + Evj.facilityVersion + "  " +
                  nexus.title + "      " +
                  Resource.get("mf.subtitle"));

    // Get the generation number and total cycles.
    
    generation = nexus.generation;
    cycles = nexus.cycles;

    // Pop the Run button if we're done, if we just paused on the
    // perfect creature, or if we just paused on convergence.
    
    if (generation >= cycles || nexus.pausePerfect || nexus.pauseConverged)
      popRunButton();

    // Set the progress bar color.

    progressBar.setForeground((isPaused()) ? pausedColor : runningColor);
    
    // Advance the progress bar.

    int percent = (int)((float)generation / cycles * 100.0f);
    progressBar.setValue((generation < cycles) ? percent : 100);

    // If we just paused on a perfect creature or on convergence,
    // say so in the progress bar.

    progressBar.setString(
      (nexus.pausePerfect) ?
        "perfect creature" :
        (nexus.pauseConverged) ? "Rseq >= Rfreq" : null);
    
    // Copy various values from the nexus into the display.
    
    generationField.setText(String.valueOf(generation));
    cyclesSpinner.setValue(new Integer(nexus.cycles));
    populationField.setText(String.valueOf(nexus.population));
    potentialField.setText(String.valueOf(nexus.potentialSites));
    sitesField.setText(String.valueOf(nexus.sites));
    widthField.setText(String.valueOf(nexus.siteWidth));
    mutationsField.setText(String.valueOf(nexus.mutations));
    rfreqField.setText(format0_00.format(nexus.Rfreq));
    selectionCheckBox.setSelected(nexus.selection);
    bestIDField.setText(String.valueOf(nexus.bestID));
    worstIDField.setText(String.valueOf(nexus.worstID));
    bestMistakesField.setText(maybeValueOf(nexus.bestMistakes));
    worstMistakesField.setText(maybeValueOf(nexus.worstMistakes));
    bestRseqField.setText(format0_00.format(nexus.bestRseq));
    worstRseqField.setText(format0_00.format(nexus.worstRseq));
    perfectionCheckBox.setSelected(nexus.perfection);
    convergenceCheckBox.setSelected(nexus.convergence);

    // Send the nexus to the logo panel and each tabbed pane. Also set
    // the titles of the tabbed panes.

    ((NexusObserver)logoPanel).acceptNexus(nexus);

    geneSeqTitleLabel.setText(tabbedPaneTitle("gs.title"));
    ((NexusObserver)geneSeqPanel).acceptNexus(nexus);

    seqLogoTitleLabel.setText(tabbedPaneTitle("sl.title"));
    ((NexusObserver)seqLogoPanel).acceptNexus(nexus);

    graphTitleLabel.setText(tabbedPaneTitle("gr.title"));
    ((NexusObserver)graphPanel).acceptNexus(nexus);

    dataStatTitleLabel.setText(tabbedPaneTitle("ds.title"));
    ((NexusObserver)dataStatPanel).acceptNexus(nexus);
  }

  // This method returns the current update frequency from the spinner.
  
  public int updateFreq ()
  {
    return ((Integer)updateSpinner.getValue()).intValue();
  }

//                      Utilities
//                      ---------


  private boolean isPaused ()
  {
    return !runButton.getModel().isSelected();
  }
  
  private String maybeValueOf (int value)
  {
    return (value >= 0) ? String.valueOf(value) : "---";
  }
  
  private void popRunButton ()
  {
    ButtonModel model = runButton.getModel();
    model.setSelected(false);
    runButton.setText("Run");
  }

  private String tabbedPaneTitle (String key)
  {
    return Resource.get(key) + " " + String.valueOf(generation);
  }
}

//                      About Box Dialog
//                      ----- --- ------


class AboutBox
{
  public static void display (Component parent)
  {
    String text = "<html>" + Evj.facilityName + " " +
                  Evj.facilityVersion +
                  Resource.get("about.text");
    
    JOptionPane.showMessageDialog(parent,
                                  text,
                                  "About " + Evj.facilityName,
                                  JOptionPane.INFORMATION_MESSAGE);
  }
}
